package com.lantone.daqe.facade;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.daqe.dto.GetByIdColumnResultDTO;
import com.lantone.daqe.dto.GetColumnResultNumberDTO;
import com.lantone.daqe.dto.GetColumnResultPageDTO;
import com.lantone.daqe.entity.ColumnResult;
import com.lantone.daqe.entity.NonnullResult;
import com.lantone.daqe.entity.RegularResult;
import com.lantone.daqe.entity.StandardvalueResult;
import com.lantone.daqe.enums.ColumnVerifyTypeEnum;
import com.lantone.daqe.facade.base.ColumnResultFacade;
import com.lantone.daqe.facade.base.NonnullResultFacade;
import com.lantone.daqe.facade.base.RegularResultFacade;
import com.lantone.daqe.facade.base.StandardvalueResultFacade;
import com.lantone.daqe.vo.GetColumnResultNumberVO;
import com.lantone.daqe.vo.GetColumnResultPageVO;
import com.lantone.daqe.vo.UpdateColumnResultVO;
import com.lantone.daqe.vo.GetByIdColumnResultVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;


/**
 * @Description: 字段校验问题明细维护API
 * @author： zhanghang
 * @time: 2022/3/8 14:49
 */
@Component
public class ColumnResultManagementFacade {
    @Autowired
    ColumnResultFacade columnResultFacade;

    @Autowired
    NonnullResultFacade nonnullResultFacade;

    @Autowired
    RegularResultFacade regularResultFacade;

    @Autowired
    StandardvalueResultFacade standardvalueResultFacade;

    /**
     * 获取字段校验问题明细分页列表数据
     *
     * @param getColumnResultPageVO
     * @return: 字段校验问题明细分页列表数据
     */
    public IPage<GetColumnResultPageDTO> getColumnResultPage(GetColumnResultPageVO getColumnResultPageVO) {
        Page<GetColumnResultPageDTO> getColumnResultPageDTOPage = new Page<>();

        QueryWrapper<ColumnResult> columnResultQueryWrapper = new QueryWrapper<>();
        columnResultQueryWrapper.eq(getColumnResultPageVO.getHospitalId() != null, "hospital_id", getColumnResultPageVO.getHospitalId());
        columnResultQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        columnResultQueryWrapper.eq(StringUtil.isNotBlank(getColumnResultPageVO.getType()), "type", getColumnResultPageVO.getType());
        columnResultQueryWrapper.eq(StringUtil.isNotBlank(getColumnResultPageVO.getIsSolved()), "is_solved", getColumnResultPageVO.getIsSolved());
        columnResultQueryWrapper.ge("solve_time", getColumnResultPageVO.getSolveTimeStart());
        columnResultQueryWrapper.le("solve_time", getColumnResultPageVO.getSolveTimeEnd());
        columnResultQueryWrapper.like(StringUtil.isNotBlank(getColumnResultPageVO.getBehospitalCode()), "behospital_code", getColumnResultPageVO.getBehospitalCode());
        columnResultQueryWrapper.like(StringUtil.isNotBlank(getColumnResultPageVO.getHosptialDatatmpCode()), "hosptial_datatmp_code", getColumnResultPageVO.getHosptialDatatmpCode());
        columnResultQueryWrapper.like(StringUtil.isNotBlank(getColumnResultPageVO.getHosptialDatatmpName()), "hosptial_datatmp_name", getColumnResultPageVO.getHosptialDatatmpName());
        columnResultQueryWrapper.like(StringUtil.isNotBlank(getColumnResultPageVO.getModeName()), "mode_name", getColumnResultPageVO.getModeName());

        Page<ColumnResult> columnResultPage = new Page<>(getColumnResultPageVO.getCurrent(), getColumnResultPageVO.getSize());
        columnResultFacade.page(columnResultPage, columnResultQueryWrapper);
        BeanUtil.copyProperties(columnResultPage, getColumnResultPageDTOPage);
        List<GetColumnResultPageDTO> getRecordTemplatePageDTOList = BeanUtil.listCopyTo(columnResultPage.getRecords(), GetColumnResultPageDTO.class);

        getColumnResultPageDTOPage.setRecords(getRecordTemplatePageDTOList);
        return getColumnResultPageDTOPage;
    }

    /**
     * 获取字段校验问题明细数量
     *
     * @param getColumnResultNumberVO
     * @return: 字段校验问题明细数量
     */
    public GetColumnResultNumberDTO getColumnResultNumber(GetColumnResultNumberVO getColumnResultNumberVO) {
        return columnResultFacade.getBaseMapper().getColumnResultNumber(getColumnResultNumberVO);
    }

    /**
     * 修改字段校验问题明细
     *
     * @param updateColumnResultVO
     * @return: 是否成功
     */
    public Boolean updateColumnResult(UpdateColumnResultVO updateColumnResultVO) {
        ColumnResult columnResultFacadeById = columnResultFacade.getById(updateColumnResultVO.getId());
        //先修改副表
        //非空校验结果表
        Long resultId = columnResultFacadeById.getResultId();
        String type = columnResultFacadeById.getType();
        if (ColumnVerifyTypeEnum.NONNULL_TYPE.getKey().equals(type)) {
            nonnullResultFacade.update(new UpdateWrapper<NonnullResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
                    .set("description", updateColumnResultVO.getDescription())
                    .set("gmt_modified", new Date())
            );
        }
        //正则校验结果表
        if (ColumnVerifyTypeEnum.REGULAR_TYPE.getKey().equals(type)) {
            regularResultFacade.update(new UpdateWrapper<RegularResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
                    .set("description", updateColumnResultVO.getDescription())
                    .set("gmt_modified", new Date())
            );
        }
        //关键字校验结果表
        if (ColumnVerifyTypeEnum.STANDARD_TYPE.getKey().equals(type)) {
            standardvalueResultFacade.update(new UpdateWrapper<StandardvalueResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
                    .set("description", updateColumnResultVO.getDescription())
                    .set("gmt_modified", new Date())
            );
        }

        return columnResultFacade.update(new UpdateWrapper<ColumnResult>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("id", updateColumnResultVO.getId())
                .set("description", updateColumnResultVO.getDescription())
                .set("gmt_modified", new Date())
        );
    }

    /**
     * 根据ID查看字段校验问题明细
     *
     * @param getByIdColumnResultVO
     * @return: 字段校验问题明细明细
     */
    public GetByIdColumnResultDTO getByIdColumnResult(GetByIdColumnResultVO getByIdColumnResultVO) {
        ColumnResult columnResult = columnResultFacade.getBaseMapper().selectById(getByIdColumnResultVO.getId());
        String type = columnResult.getType();
        GetByIdColumnResultDTO getByIdColumnResultDTO = new GetByIdColumnResultDTO();
        //非空校验
        if (ColumnVerifyTypeEnum.NONNULL_TYPE.getKey().equals(type)) {
            getByIdColumnResultDTO.setType(type);
            Long resultId = columnResult.getResultId();
            NonnullResult nonnullResult = nonnullResultFacade.getBaseMapper().selectOne(new QueryWrapper<NonnullResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
            );
            BeanUtil.copyProperties(nonnullResult, getByIdColumnResultDTO);
            return getByIdColumnResultDTO;
        }
        //正则校验结果表
        if (ColumnVerifyTypeEnum.REGULAR_TYPE.getKey().equals(type)) {
            getByIdColumnResultDTO.setType(type);
            Long resultId = columnResult.getResultId();
            RegularResult regularResult = regularResultFacade.getBaseMapper().selectOne(new QueryWrapper<RegularResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
            );
            BeanUtil.copyProperties(regularResult, getByIdColumnResultDTO);
            return getByIdColumnResultDTO;
        }
        //关键字校验结果表
        if (ColumnVerifyTypeEnum.STANDARD_TYPE.getKey().equals(type)) {
            getByIdColumnResultDTO.setType(type);
            Long resultId = columnResult.getResultId();
            StandardvalueResult standardvalueResult = standardvalueResultFacade.getBaseMapper().selectOne(new QueryWrapper<StandardvalueResult>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("id", resultId)
            );
            BeanUtil.copyProperties(standardvalueResult, getByIdColumnResultDTO);
            return getByIdColumnResultDTO;
        }
        return getByIdColumnResultDTO;
    }
}
